/**************************************************************************
 *  This file is part of QXmlEdit                                         *
 *  Copyright (C) 2011-2018 by Luca Bellonda and individual contributors  *
 *    as indicated in the AUTHORS file                                    *
 *  lbellonda _at_ gmail.com                                              *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Library General Public            *
 * License as published by the Free Software Foundation; either           *
 * version 2 of the License, or (at your option) any later version.       *
 *                                                                        *
 * This library is distributed in the hope that it will be useful,        *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 * Library General Public License for more details.                       *
 *                                                                        *
 * You should have received a copy of the GNU Library General Public      *
 * License along with this library; if not, write to the                  *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA  02110-1301  USA                                            *
 **************************************************************************/

#ifndef LIBQXMLEDIT_GLOBAL_H
#define LIBQXMLEDIT_GLOBAL_H

#include <QtCore/qglobal.h>

#if not defined(LIBQXMLEDIT_LIBRARY_STATIC)

# if defined(LIBQXMLEDIT_LIBRARY)
#   define LIBQXMLEDITSHARED_EXPORT Q_DECL_EXPORT
# else
#   define LIBQXMLEDITSHARED_EXPORT Q_DECL_IMPORT
# endif

#else
#   define LIBQXMLEDITSHARED_EXPORT
#endif

#endif // LIBQXMLEDIT_GLOBAL_H
